// Need G4P library
import g4p_controls.*;
import processing.serial.*;
import java.text.DecimalFormat;

PGraphics pg;
int dispWidth = 640;
int dispHeight = 380;

Serial port;
byte data[] = new byte[128];

byte frequency = 2;
byte triggerHigh = 1;
boolean settingsNeedUpdate = true;

float zoom = 1;

public void setup(){
  size(640, 480, JAVA2D);
  createGUI();
  customGUI();
  
  println(Serial.list());
  port = new Serial(this, "/dev/tty.usbmodemfa131", 115200);
  
  waitForOK();
  
  delay(100);
}

public void update() {
  if (settingsNeedUpdate) {
    data[0] = 66;
    data[1] = frequency;
    println(data[1]);
    port.write(data[0]);
    port.write(data[1]);
    port.write(triggerHigh);
    waitForOK();
    settingsNeedUpdate = false;
    getStatus();
  }
  
  port.write(65);
  int i = 0;
  int timeout = 400;
  while (i < 128) {
    while (port.available() > 0)
      data[i++] = (byte)port.read();
    delay(1);
    if (--timeout <= 0) {
      while (port.available() > 0)
        port.read();
      break;
    }
  }
}

public void renderDisp() {
  pg.beginDraw();
  pg.background(0);
  
  pg.stroke(0,255,0);
  float xPerSample = dispWidth / (float)1024.0 * zoom;
  float x = 0;
  int y = dispHeight / 2;
  int yOffset;
  PVector lp = new PVector(x, y);
  
  
  for (int i = 0; i < 1024; ++i) {
    int test = (byte)((data[i / 8] >> (7 - i%8)) & 0x01);
    if (test == 1)
      yOffset = -100;
    else
      yOffset = 100;
    PVector np = new PVector((int)x, y + yOffset);
    pg.line(lp.x, lp.y, np.x, np.y);
    lp = np;
    x += xPerSample;
  }
  
  pg.stroke(255);
  int samples = 1024;
  int dispSamples = (int)(samples / zoom);
  int dispUS = dispSamples / frequency;
  int numLabels;
  int labelInc;
  if (dispUS > 400)
    labelInc = 50;
  else if (dispUS > 200)
    labelInc = 20;
  else if (dispUS > 100)
    labelInc = 10;
  else if (dispUS > 50)
    labelInc = 5;
  else if (dispUS > 20)
    labelInc = 2;
  else
    labelInc = 1;
  
  float ticksPerLabel, numTicks;
  ticksPerLabel = 5;
  
  numLabels = dispUS / labelInc;
  numTicks = numLabels * ticksPerLabel;
  
  x = 0;
  for (int i = 0; i < numLabels; i++) {
    drawTick(x, true);
    pg.text(i * labelInc, x-4, dispHeight - 10);
    x += dispWidth / (float)numLabels;
  }
  
  x = 0;
  for (int i = 0; i < numTicks; i++) {
    if (!(i % ticksPerLabel == 0))
      drawTick(x, false);
    x += dispWidth / (float)numTicks;
  }
  
  if (mouseY < 380) {
    pg.line(mouseX, 0, mouseX, dispHeight);
    pg.text(new DecimalFormat("###0.0").format((dispUS/(float)dispWidth * mouseX)) + "us", mouseX + 5, 15);
  }
  
  //pg.text(frameRate, 10, 10);
  pg.endDraw();
}

public void draw(){
  update();
  renderDisp();
}

public void drawTick(float x, boolean ext) {
  pg.line(x, dispHeight-40 + (ext ? 8 : 0), x, dispHeight - 45);
}

public void waitForOK() {
  while (true) {
    if (port.available() > 0)
      if (port.read() == 1)
        break;
    delay(10);
  }
}

public void getStatus() {
  port.write(67);
  String text = "";
  out: while (true) {
    while (port.available() > 0) {
      char a = port.readChar();
      if ((int)a == 1)
        break out;
      print(a);
    }
    delay(10);
  }
      
  print(text);
}

// Use this method to add additional statements
// to customise the GUI controls
public void customGUI(){
  pg = createGraphics(dispWidth, dispHeight);
  disp.setGraphic(pg);
}
